/* 
 * APSR RIPv2 Daemon, version 0.1
 * by arbon@gmx.de, 01 May 2005
 *
 * -bash-2.05b# gcc -Wall -O2 apsr_ripv2_daemon.c  -o apsr_ripv2_daemon -lapsr
 * -bash-2.05b# ./apsr_ripd eth0 192.168.0.43 12.0.0.0 255.0.0.0 1
 * Info : apsr_init_psize malloc succeeded!
 * Sending update number 0 for network 12.0.0.0, next in 1 seconds.
 * Sending update number 1 for network 12.0.0.0, next in 1 seconds.
 * ...
 *
 */

#include <apsrlib/apsrlib.h>

void usage () {
  printf("Usage: ./apsr-ripd <device> <src ip> <network> <mask> <update timer in secs>\n");
  exit(0);
}

int main (int argc, char *argv[]) {
	struct apsr_link_layer *apsr_ll;
	char *device, *src_ip, *network, *mask;
        
	unsigned char src_mac[6] = {0x00, 0x00, 0x0a, 0x0b, 0x0c, 0x0d};
	unsigned char dst_mac[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};

	int my_ret, written, update, counter = 0;
	unsigned char *buf;

        if (argc < 5) {
          usage();
	}

	device = argv[1];
        src_ip = argv[2];
        network = argv[3];
        mask = argv[4];
        update = atoi(argv[5]);

	/* Open a link layer interface. */
	apsr_ll = (struct apsr_link_layer *) apsr_open_link_layer(device);

	/* Initialize the packet memory. */
	/* We need 14 bytes for the ethernet header, 20 bytes */
	/* for the IPv4 header and 8 bytes for the UDP header. */
	my_ret = apsr_init_psize(APSR_ETH_H + APSR_IPv4_H + APSR_UDP_H + APSR_RIP_H, &buf);

	if(my_ret == 0 || buf == NULL) ERROR("apsr_init_psize failed!");

	/* Build test packet */
	/* Firstly we construct the ethernet header. */
	my_ret = apsr_ethernet(dst_mac,		// Destination address
		     	       src_mac,		// Source address
			       ETHERTYPE_IP,	// Ethernet type
		   	       NULL,		// Payload
			       0,		// Payload size
			       buf);		// Buffer

	if(my_ret == -1) ERROR("apsr_ethernet failed!");

	/* After that we construct a IPv4 packet. */
 	my_ret = apsr_ipv4(4,			// IP version
			   5,			// IP header length
			   APSR_UDP_H+APSR_RIP_H,// IP length
			   0x00C0,		// IP Type of Service (ToS)
			   0,			// IP Identification (ID)
			   0x0000,		// IP Fragementation offset
			   2,			// IP Time to live (TTL)
			   APSR_PROTO_UDP,	// IP transport layer protocol
			   inet_addr(src_ip), 	// Source IP
			   inet_addr("224.0.0.9"), // Destination IP
			   0,			// IP checksum
			   NULL,		// IP options
			   0,			// IP option size
			   NULL,		// IP payload
			   0,			// IP payload length
			   buf + APSR_ETH_H);	// Packet buffer

	if(my_ret == -1) ERROR("apsr_ipv4 failed!");

	/* And last, but not least the UDP packet. */
	my_ret = apsr_udp(520,			// UDP source port
			  520,			// UDP destination port
			  32,			// Length (8 Byte, no payload)
			  0x0000,		// Checksum (calculated later)
			  NULL,			// Payload
			  0,			// Payload size
			  buf + APSR_ETH_H
			      + APSR_IPv4_H);	// Packet buffer
	
	if(my_ret == -1) ERROR("apsr_udp failed!");

	/* RIPv2 Response. */ 
	apsr_rip(APSR_RIP_RESPONSE,		// RIP command
		 2,				// RIP version
		 0x0000,			// RIP routing domain
		 APSR_RIP_IDENT_IP,		// RIP address family
		 0x0000,			// RIP route tag
		 inet_addr(network),		// RIP IP address
		 inet_addr(mask),		// RIP subnet mask
		 0x0000,			// RIP next hop IP
		 0x0001,			// RIP metric
		 NULL,				// Payload
		 0,				// Payload size
		 buf + APSR_ETH_H +
		 APSR_IPv4_H + APSR_UDP_H);	// Packet buffer

        /* Calculate the checksum */
        apsr_checksum(buf+APSR_ETH_H, APSR_PROTO_UDP, APSR_UDP_H + APSR_RIP_H);

	/* Never ending story... */
        for (counter = 1; counter >= -1; counter++) {
          /* Send it */
          written = apsr_write_link_layer(apsr_ll, device, buf,
                                        APSR_ETH_H + APSR_IPv4_H +
                                        APSR_UDP_H + APSR_RIP_H, 1);

    	  apsr_output("Sending update number %d for network %s/%s, next in %d seconds.\n", counter, network, mask, update);

	  /* Sleep for n-seconds */ 
          sleep(update);
        }	

	/* Close link layer */
	apsr_close_link_layer(apsr_ll);
	
	return(RET_SUCCESS);
}

